﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.DragDrop;
using System.Collections;
using System.Collections.ObjectModel;

namespace SL.DragListBox
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            RadDragAndDropManager.AddDropQueryHandler(listBox, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
            RadDragAndDropManager.AddDragQueryHandler(listBox, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
            RadDragAndDropManager.AddDropInfoHandler(listBox, new EventHandler<DragDropEventArgs>(OnDropInfo));
            RadDragAndDropManager.AddDragInfoHandler(listBox, new EventHandler<DragDropEventArgs>(OnDragInfo));

            listBox.ItemsSource = new ObservableCollection<CheckableItemsViewModel>(from c in Enumerable.Range(0,300) select new CheckableItemsViewModel{IsChecked=true,Name="Name"+c});
        }

        // OnDragQuery event handler
        private void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            var listBoxItem = e.Options.Source as ContentControl;
            if (e.Options.Status == DragStatus.DragQuery && listBoxItem != null)
            {
                var sourceControl = e.Options.Source;
                var dragCue = RadDragAndDropManager.GenerateVisualCue(sourceControl);
                dragCue.HorizontalAlignment = HorizontalAlignment.Left;
                dragCue.Content = sourceControl.DataContext;
                dragCue.ContentTemplate = listBoxItem.ContentTemplate;
                e.Options.DragCue = dragCue;
                e.Options.Payload = new DragDropOperation()
                {
                    Payload = sourceControl.DataContext
                };
            }
            e.QueryResult = true;
            e.Handled = true;
        }

        // OnDragInfo event handler
        private void OnDragInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Status == DragStatus.DragComplete)
            {
                var listBox = e.Options.Source.FindItemsConrolParent() as ItemsControl;
                var itemsSource = listBox.ItemsSource as IList;
                var operation = e.Options.Payload as DragDropOperation;
                itemsSource.Remove(operation.Payload);
            }
        }

        // OnDropQuery event handler
        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            var destination = e.Options.Destination;
            var operation = e.Options.Payload as DragDropOperation;
            if (e.Options.Status == DragStatus.DropDestinationQuery && destination is ListBoxItem && operation != null)
            {
                var listBox = destination.FindItemsConrolParent() as ListBox;
                //Cannot place an item relative to itself:
                if (e.Options.Source == e.Options.Destination)
                {
                    return;
                }
                //Get the spatial relation between the destination item and the vis. root:
                var destinationTopLeft = destination.TransformToVisual(null).Transform(new Point());
                //Should the new Item be moved before or after the destination item?:
                bool placeBefore = (e.Options.CurrentDragPoint.X - destinationTopLeft.X) < destination.ActualWidth / 2;
                operation.DropPosition = placeBefore ? DropPosition.Before : DropPosition.After;
                e.QueryResult = true;
                e.Handled = true;
            }
            if (e.Options.Status == DragStatus.DropDestinationQuery && destination is ListBox && operation != null)
            {
                var listBox = destination as ListBox;
                var itemsCollection = listBox.ItemsSource.Cast<object>();
                // Cannot drop the last or only item of the list box within the same list box:
                if (listBox.ItemsSource != null && (!itemsCollection.Any() || itemsCollection.Last() != operation.Payload))
                {
                    e.QueryResult = true;
                    e.Handled = true;
                }
                else
                {
                    e.QueryResult = false;
                    e.Handled = true;
                }
            }
        }

        // OnDropInfo event handler
        private void OnDropInfo(object sender, DragDropEventArgs e)
        {
            var destination = e.Options.Destination;
            if (e.Options.Status == DragStatus.DropPossible && destination is ListBoxItem)
            {
                var listBox = destination.FindItemsConrolParent() as ListBox;
                VisualStateManager.GoToState(listBox, "DropPossible", false);
                //Get the DropCueElemet:
                var dropCueElement = (VisualTreeHelper.GetChild(listBox, 0) as FrameworkElement).FindName("DropCueElement") as FrameworkElement;
                var operation = e.Options.Payload as DragDropOperation;
                //Get the parent of the destination:
                var visParent = VisualTreeHelper.GetParent(destination) as UIElement;
                //Get the spacial relation between the destination and its parent:
                var destinationStackTopLeft = destination.TransformToVisual(visParent).Transform(new Point());
               // var yTranslateValue = operation.DropPosition == DropPosition.Before ? destinationStackTopLeft.Y : destinationStackTopLeft.Y + destination.ActualHeight;
                var xTranslateValue = operation.DropPosition == DropPosition.Before ? destinationStackTopLeft.X : destinationStackTopLeft.X + destination.ActualWidth;

                dropCueElement.RenderTransform = new TranslateTransform()
                {
                    Y = destinationStackTopLeft.Y,
                    X = xTranslateValue
                };
                e.Handled = true;
            }
            if (e.Options.Status == DragStatus.DropPossible && destination is ListBox)
            {
                var listBox = destination as ListBox;
                VisualStateManager.GoToState(listBox, "DropPossible", false);
                //Get the DropCueElemet:
                var dropCueElement = (VisualTreeHelper.GetChild(listBox, 0) as FrameworkElement).FindName("DropCueElement") as FrameworkElement;
                var operation = e.Options.Payload as DragDropOperation;
                // Get the size of the items:
                var itemsPresenter = listBox.GetTemplateChild<FrameworkElement>("ItemsPresenterElement");
                var panel = VisualTreeHelper.GetChild(itemsPresenter, 0) as Panel;
                if (panel != null)
                {
                    var yTranslateValue = panel.ActualHeight;
                    dropCueElement.RenderTransform = new TranslateTransform()
                    {
                        Y = yTranslateValue
                    };
                }
            }
            //Hide the DropCue:
            if (e.Options.Status == DragStatus.DropImpossible || e.Options.Status == DragStatus.DropCancel || e.Options.Status == DragStatus.DropComplete)
            {
                var listBox = destination as ListBox;
                if (listBox == null)
                {
                    listBox = e.Options.Destination.FindItemsConrolParent() as ListBox;
                }
                VisualStateManager.GoToState(listBox, "DropImpossible", false);
            }
            //Place the item:
            if (e.Options.Status == DragStatus.DropComplete && destination is ListBoxItem)
            {
                var listBox = e.Options.Destination.FindItemsConrolParent() as ListBox;
                var itemsSource = listBox.ItemsSource as IList;
                var destinationIndex = itemsSource.IndexOf(e.Options.Destination.DataContext);
                var operation = e.Options.Payload as DragDropOperation;
                var insertIndex = operation.DropPosition == DropPosition.Before ? destinationIndex : destinationIndex + 1;
                itemsSource.Insert(insertIndex, operation.Payload);
                listBox.Dispatcher.BeginInvoke(new Action(() =>
                {
                    listBox.SelectedIndex = insertIndex;
                }));
            }
            if (e.Options.Status == DragStatus.DropComplete && destination is ListBox)
            {
                var listBox = destination as ListBox;
                var itemsSource = listBox.ItemsSource as IList;
                var operation = e.Options.Payload as DragDropOperation;
                itemsSource.Add(operation.Payload);
                listBox.Dispatcher.BeginInvoke(new Action(() =>
                {
                    listBox.SelectedIndex = itemsSource.Count - 1;
                }));
            }
        }


    }
}
